package edu.ouc.mycloud.controller;

import java.util.Locale;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.logout.SecurityContextLogoutHandler;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

/**
 * Handles requests for the application home page.
 */
@Controller
public class HomeController extends MainController {
	
	private static final Logger logger = LoggerFactory.getLogger(HomeController.class);
	
	/**
	 * Simply selects the home view to render by returning its name.
	 */
	@RequestMapping(value = "/", method = RequestMethod.GET)
	public String home(Locale locale, Model model) {
		
		
		if (this.getPrincipal().getRol().getIsAdministrator())
		{
			model.addAttribute("userNameAuthenticated", this.getPrincipal());
			model.addAttribute("spaceUsed", this.getSpaceUserd());
			model.addAttribute("numberOfRoles", this.getNumberOfRoles());
			model.addAttribute("numberOfUsers", this.getNumberOfUsers());
			model.addAttribute("numberOfVirtualFiles", this.getNumberOfVirtualFiles());
			model.addAttribute("numberOfFiles", this.getNumberOfFiles());
			return "/home/home";
		}
		else
		{
			return "redirect:/storage";
		}		
	}
	
	@RequestMapping(value = "/login", method = RequestMethod.GET)
	public String loginPage(Locale locale, Model model) {
		return "login";
	}
	
	@RequestMapping(value="/logout", method = RequestMethod.GET)
	public String logoutPage (HttpServletRequest request, HttpServletResponse response) {
	    Authentication auth = SecurityContextHolder.getContext().getAuthentication();
	    if (auth != null){    
	        new SecurityContextLogoutHandler().logout(request, response, auth);
	    }
	    return "redirect:/login?logout";
	}
	
	@RequestMapping(value = "/page404", method = RequestMethod.GET)
	public String ErrorPage404(Locale locale, Model model) {
		return "page404";
	}	
	
	@RequestMapping(value = "/page403", method = RequestMethod.GET)
	public String ErrorPage403(Locale locale, Model model) {
		return "page403";
	}	
}
