package edu.ouc.mycloud.controller;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.URLConnection;
import java.net.UnknownHostException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.util.FileCopyUtils;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.multipart.MultipartFile;

import edu.ouc.mycloud.model.File;
import edu.ouc.mycloud.model.Rol;
import edu.ouc.mycloud.model.Shared;
import edu.ouc.mycloud.model.StoredFile;
import edu.ouc.mycloud.model.User;
import edu.ouc.mycloud.service.StorageService;
import edu.ouc.mycloud.service.UserService;

@Controller
public class StorageController extends MainController {

	private static final Logger logger = LoggerFactory.getLogger(MainController.class);
	
	@Autowired
	private StorageService storageService;
	
	@Autowired
	private UserService userService;
	
	@InitBinder
	protected void initBinder(WebDataBinder binder) {
	    SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy");
	    binder.registerCustomEditor(Date.class, new CustomDateEditor(
	            dateFormat, true));
	}
	
	@RequestMapping(value = "/storage/saveprivateshared", method = RequestMethod.POST)
	public String updatePrivateFile(@ModelAttribute("file") File file) {
		File fileAux = this.storageService.getFileById(file.getId());
		User user = this.userService.getUserById(file.getUser().getId());
		
		File newFile = new File();
		newFile.setFilename(fileAux.getFilename());
		newFile.setIsFolder(false);
		newFile.setParentfile(null);
		newFile.setStoreddate(new Date());
		newFile.setStoredfile(fileAux.getStoredfile());
		newFile.setUser(user);
		
		this.storageService.addFile(newFile);
		return "redirect:/storage";	
	}
	
	private Map<Integer, String> GetUsersForSelect() {
		User loggedUser = this.getPrincipal();
		Map<Integer,String> userList = new LinkedHashMap<Integer,String>();
		userList.put(-1, "");
		for (User user : this.userService.listUsers())
		{
			if (user.getId()!=loggedUser.getId() )
			{
				userList.put(user.getId(), user.getName()+","+user.getSurname());
			}
		}
		return userList;
	}
	
	@RequestMapping(value = "/storage/sharedfilewithmember/{id}", method = RequestMethod.GET)
	public String sharedFileWithMember(@PathVariable("id") int id, Model model) {
		File file = this.storageService.getFileById(id);
		file.setUser(null);
		
		Map<Integer, String> userList = GetUsersForSelect();		
		model.addAttribute("userList", userList);
		model.addAttribute("file", file);
		model.addAttribute("userNameAuthenticated", this.getPrincipal());
		model.addAttribute("spaceUsed", this.getSpaceUserd());
		return "storage/sharedfilewithmember";
	}
	
	
	@RequestMapping(value = "/storage/verifyDownloadFile", method = RequestMethod.POST)
	public String verifyDownloadFile(@ModelAttribute("shared") Shared sharedFile,@ModelAttribute("password") String password, Model model, HttpServletResponse response) {
		Shared shared = this.storageService.getFileByToken(sharedFile.getToken());
		if ((shared.getDeadline()==null) || (shared.getDeadline().compareTo(new Date())==1))
		{
			if (shared.getPassword().equals(password))
			{			
				String mimeType= URLConnection.guessContentTypeFromName(shared.getFile().getFilename());
				if(mimeType==null){
					mimeType = "application/octet-stream";
				}
				response.setContentType(mimeType);
				response.setHeader("Content-Disposition", String.format("inline; filename=\"" + shared.getFile().getFilename() +"\""));
				response.setContentLength((int)shared.getFile().getStoredfile().getFilesize());
			
				InputStream inputStreamFile = this.storageService.getStoredPhysicalFile(shared.getFile().getId());
			
				try {
					FileCopyUtils.copy(inputStreamFile, response.getOutputStream());
					response.flushBuffer();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				return null;
			}
			else
			{
				throw new FileNotFoundException();
			}		
		}
		else
		{
			throw new FileNotFoundException();
		}
	}
	
	
	@RequestMapping(value = "/storage/saveshared", method = RequestMethod.POST)
	public String saveFile(@ModelAttribute("shared") Shared shared, Model model, HttpServletRequest request) {
		String internalToken = java.util.UUID.randomUUID().toString().replaceAll("-", "").substring(0, 32);
		shared.setToken(internalToken);
		this.storageService.setSharedFile(shared);		
		model.addAttribute("userNameAuthenticated", this.getPrincipal());
		model.addAttribute("spaceUsed", this.getSpaceUserd());
				
		StringBuilder sbUrl = new StringBuilder();
		sbUrl.append(String.format("%s://%s:%d%s/",request.getScheme(),  request.getServerName(), request.getServerPort(),request.getContextPath()));		
		sbUrl.append("storage/downloadfile/");
		sbUrl.append(shared.getToken());
		
		StringBuilder sbMail = new StringBuilder();
		sbMail.append("name@uoc.edu");
		sbMail.append("?");
		sbMail.append("subject");
		sbMail.append("=");
		sbMail.append("Quiero compartir este fichero contigo.");
		sbMail.append("&");
		sbMail.append("body");
		sbMail.append("=");
		sbMail.append(sbUrl.toString());
		
		model.addAttribute("filename", shared.getFile().getFilename());
		model.addAttribute("sendemail", sbMail.toString());
		model.addAttribute("sbUrl", sbUrl.toString());
		return "storage/endsharedfile";
	}
	
	@RequestMapping(value = "/storage/sharedfile/{id}", method = RequestMethod.GET)
	public String sharedFile(@PathVariable("id") int id, Model model) {
		File file = this.storageService.getFileById(id);
		Shared shared = new Shared();
		shared.setFile(file);
		model.addAttribute("shared", shared);
		model.addAttribute("userNameAuthenticated", this.getPrincipal());
		model.addAttribute("spaceUsed", this.getSpaceUserd());
		return "storage/sharedfile";
	}
	
	@RequestMapping(value="/storage/downloadfile/{token}", method = RequestMethod.GET)
	public String downloadPublicFile(HttpServletResponse response,@PathVariable("token") String token, Model model) {
		Shared shared = this.storageService.getFileByToken(token);
		
		if(shared.getPassword().isEmpty())
		{
			if ((shared.getDeadline()==null) || (shared.getDeadline().compareTo(new Date())==1))
			{
				String mimeType= URLConnection.guessContentTypeFromName(shared.getFile().getFilename());
				if(mimeType==null){
					mimeType = "application/octet-stream";
				}
				response.setContentType(mimeType);
				response.setHeader("Content-Disposition", String.format("inline; filename=\"" + shared.getFile().getFilename() +"\""));
				response.setContentLength((int)shared.getFile().getStoredfile().getFilesize());
			
				InputStream inputStreamFile = this.storageService.getStoredPhysicalFile(shared.getFile().getId());
			
				try {
					FileCopyUtils.copy(inputStreamFile, response.getOutputStream());
					response.flushBuffer();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				return null;
			}
			else
			{
				throw new FileNotFoundException();
			}
		}
		else
		{
			model.addAttribute("shared", shared);
			return "/storage/downloadfilewithpass";
		}		
	}
	
	@ResponseStatus(value=HttpStatus.NOT_FOUND, reason="Fichero no encontrado")  // 404
	 public class FileNotFoundException extends RuntimeException {
	 }
	
	@RequestMapping(value="/storage/download/{id}", method = RequestMethod.GET)
    public void downloadFile(HttpServletResponse response, @PathVariable("id") int id) throws IOException {
		File file = this.storageService.getFileById(id);
		User loggedUser = this.getPrincipal();
		
		if ((file.getUser().getId()==loggedUser.getId()) || (loggedUser.getRol().getIsAdministrator()))
		{			
			String mimeType= URLConnection.guessContentTypeFromName(file.getFilename());
			if(mimeType==null){
	            mimeType = "application/octet-stream";
	        }
			response.setContentType(mimeType);
			response.setHeader("Content-Disposition", String.format("inline; filename=\"" + file.getFilename() +"\""));
			response.setContentLength((int)file.getStoredfile().getFilesize());
			
			InputStream inputStreamFile = this.storageService.getStoredPhysicalFile(file.getId());
			
			FileCopyUtils.copy(inputStreamFile, response.getOutputStream());
			response.flushBuffer();
		}
		else 
		{
			throw new FileNotFoundException();
		}			
	}
	
	
			
	@RequestMapping(value = "/storage/adddirectory/{id}", method = RequestMethod.GET)
	public String newDirectory(@PathVariable("id") int id, Model model) {
		File fileAux = this.storageService.getFileById(id);
		File file = new File();
		file.setParentfile(fileAux);
		model.addAttribute("file", file);
		model.addAttribute("userNameAuthenticated", this.getPrincipal());
		model.addAttribute("spaceUsed", this.getSpaceUserd());
		return "storage/addsubdirectory";
	}
	
	@RequestMapping(value = "/storage/addfile", method = RequestMethod.GET)
	public String newFile(Model model) {
		model.addAttribute("file", new File());
		model.addAttribute("userNameAuthenticated", this.getPrincipal());
		model.addAttribute("spaceUsed", this.getSpaceUserd());
		return "storage/addfile";
	}
	
	@RequestMapping(value = "/storage/addfile/{id}", method = RequestMethod.GET)
	public String newFile(@PathVariable("id") int id, Model model) {
		model.addAttribute("parentid", id);
		model.addAttribute("file", new File());
		model.addAttribute("userNameAuthenticated", this.getPrincipal());
		model.addAttribute("spaceUsed", this.getSpaceUserd());
		return "storage/addfileindirectory";
	}
	
	@RequestMapping("/storage/rename/{id}")
    public String editUser(@PathVariable("id") int id, Model model){
        model.addAttribute("file", this.storageService.getFileById(id));
        model.addAttribute("userNameAuthenticated", this.getPrincipal());
        model.addAttribute("spaceUsed", this.getSpaceUserd());
        return "storage/rename";
    }
	
	@RequestMapping(value = "/storage/update", method = RequestMethod.POST)
	public String update(@ModelAttribute("file") File file) {
		File fileAux = this.storageService.getFileById(file.getId());
		fileAux.setFilename(file.getFilename());
		this.storageService.updateFile(fileAux);
		return "redirect:/storage";	
	}
	
	@RequestMapping(value = "/storage/savefile", method = RequestMethod.POST)
	public String saveFile(@ModelAttribute("document") MultipartFile document, Model model) {
		if (!document.isEmpty())
		{
			return saveNewFile(document,null);
		}
		else
		{
			return "redirect:/storage/addfile?error=1";
		}
	}
	
	@RequestMapping(value = "/storage/savefile/{id}", method = RequestMethod.POST)
	public String saveFile(@ModelAttribute("document") MultipartFile document, @PathVariable("id") int id, Model model) {
		if (!document.isEmpty())
		{
			return saveNewFile(document,id);	
		}
		else
		{
			return "redirect:/storage/addfile/"+id+"?error=1";
		}
	}
	
	@RequestMapping(value = "/storage/{id}", method = RequestMethod.GET)
	public String listFilesByParentFile(@PathVariable("id") int id, Model model) {	
		
		File file = this.storageService.getFileById(id);
		List<File> breadcrumb = new ArrayList<File>();
		
		while (file!=null ) {
			breadcrumb.add(0,file);
			file = file.getParentfile();
		}
		
		User loggedUser = this.getPrincipal();
		List<File> files = null;
		if (loggedUser.getRol().getIsAdministrator())
		{
			files = this.storageService.listFilesByParentFile(id);	
		}
		else
		{
			files = this.storageService.listFilesByParentFile(id, loggedUser);	
		}
		
		model.addAttribute("breadcrumb", breadcrumb);
		model.addAttribute("listFiles", files);
		model.addAttribute("userNameAuthenticated", this.getPrincipal());
		model.addAttribute("spaceUsed", this.getSpaceUserd());
		return "storage/list";
	}	
	
	@RequestMapping(value = "/storage", method = RequestMethod.GET)
	public String listFiles(Model model) {
		
		User loggedUser = this.getPrincipal();
		List<File> files = null;
		if (loggedUser.getRol().getIsAdministrator())
		{
			files = this.storageService.listFiles();	
		}
		else
		{
			files = this.storageService.listFiles(loggedUser);	
		}	
		List<File> breadcrumb = new ArrayList<File>();			
		model.addAttribute("breadcrumb", breadcrumb);
		model.addAttribute("listFiles", files);
		model.addAttribute("userNameAuthenticated", this.getPrincipal());
		model.addAttribute("spaceUsed", this.getSpaceUserd());
		return "storage/list";
	}
		
	@RequestMapping(value = "/storage/adddirectory", method = RequestMethod.GET)
	public String newDirectory(Model model) {
		model.addAttribute("file", new File());
		model.addAttribute("userNameAuthenticated", this.getPrincipal());
		model.addAttribute("spaceUsed", this.getSpaceUserd());
		return "storage/adddirectory";
	}
	
	@RequestMapping(value = "/storage/savedirectory", method = RequestMethod.POST)
	public String save(@ModelAttribute("file") File file) {
		User user = this.getPrincipal();
		file.setStoreddate(new Date());
		file.setIsFolder(true);
		file.setUser(user);		
		this.storageService.addFile(file);
		String pathToReturn = "";
		if (file.getParentfile()==null)
		{
			pathToReturn = "redirect:/storage";
		}
		else
		{
			pathToReturn = "redirect:/storage/"+file.getParentfile().getId();
		}
		return pathToReturn;	
	}
		
	@RequestMapping("/storage/remove/{id}")
    public String removeFile(@PathVariable("id") int id, Model model){	
		File file = this.storageService.getFileById(id);
		String pathToReturn = "";
        if (file.getParentfile()==null)
		{
			pathToReturn = "redirect:/storage";
		}
		else
		{
			pathToReturn = "redirect:/storage/"+file.getParentfile().getId();
		}        
        if (this.storageService.removeFile(id))
        {
        	//TODO
        }
		return pathToReturn;
    }
		
	private String saveNewFile(MultipartFile document, Integer id) {
				
		// Define el nombre originel del fichero		
		String fullPath = document.getOriginalFilename();
		int index = fullPath.lastIndexOf("\\");
		String fileName = fullPath.substring(index + 1);
		
		// Define el nombre interno
		String internalFileName = java.util.UUID.randomUUID().toString().replaceAll("-", "").substring(0, 32);
		
		User user = this.getPrincipal();
		
		// Se definene el fichero
		File file = new File();
		file.setStoreddate(new Date());
		file.setIsFolder(false);
		file.setUser(user);								
		file.setFilename(fileName);
		 		
		// Se definene el fichero almacenado
		StoredFile storedFile = new StoredFile();
		storedFile.setFilename(internalFileName);
		storedFile.setFilesize(document.getSize());		
		file.setStoredfile(storedFile);
		
		Integer quota = this.getPrincipal().getRol().getQuota();
		
		int error=0;
		
		if (!document.isEmpty())
		{
			// Existe padre
			if (id!=null)
			{
				File parentfile = this.storageService.getFileById(id);				
				file.setParentfile(parentfile);
				if (parentfile.getUser().getId()!=user.getId())
				{
					error=2;
				}
				else
				{
					if (quota!=null)
					{
					
						if ((document.getSize()/(1024*1024)+this.getSpaceUserd())>quota)
						{
							error=3;
						}
						else
						{
							this.storageService.addFile(file, document);
						}	
					}
					else
					{
						this.storageService.addFile(file, document);
					}
				}
			}
			else
			{
				if (quota!=null)
				{
					if ((document.getSize()/(1024*1024)+this.getSpaceUserd())>quota)
					{
						error=3;
					}
					else
					{
						this.storageService.addFile(file, document);
					}
				}
				else
				{
					this.storageService.addFile(file, document);
				}
			}
		}
						
		String pathToReturn = "";
		if (file.getParentfile()==null)
		{
			pathToReturn = "redirect:/storage";
		}
		else
		{
			if (error>0)
			{
				pathToReturn = "redirect:/storage/addfile/"+file.getParentfile().getId()+"?error="+error;
			}
			else
			{
				pathToReturn = "redirect:/storage/"+file.getParentfile().getId();
			}
		}		
		return pathToReturn;
	}

}
