package edu.ouc.mycloud.controller;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

import org.apache.taglibs.standard.tag.common.xml.ForEachTag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import edu.ouc.mycloud.model.Rol;
import edu.ouc.mycloud.model.User;
import edu.ouc.mycloud.modelview.UserPass;
import edu.ouc.mycloud.service.RolService;
import edu.ouc.mycloud.service.UserService;

@Controller
public class UsersController extends MainController {

	@Autowired
	private UserService userService;
	
	@Autowired
	private RolService rolService;
	
	@InitBinder
	protected void initBinder(WebDataBinder binder) {
	    SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy");
	    binder.registerCustomEditor(Date.class, new CustomDateEditor(
	            dateFormat, false));
	}
	
	@Autowired(required=true)
	@Qualifier(value="rolService")
	public void setRolService(RolService rolService){
		this.rolService = rolService;
	}
	
	@RequestMapping(value = "/users", method = RequestMethod.GET)
	public String listUsers(Model model) {
		model.addAttribute("listUsers", this.userService.listUsers());
		model.addAttribute("userNameAuthenticated", this.getPrincipal());
		model.addAttribute("spaceUsed", this.getSpaceUserd());
		return "users/list";
	}
	
	
	
	@RequestMapping(value = "/users/update", method = RequestMethod.POST)
	public String update(@ModelAttribute("user") User user) {
		this.userService.updateUser(user);
		return "redirect:/users";	
	}
	
	
	
	
	
	
	
	
	
	@RequestMapping("/users/edituser/{id}")
    public String editUserFromUser(@PathVariable("id") int id, Model model){
        model.addAttribute("user", this.userService.getUserById(id));
        model.addAttribute("userNameAuthenticated", this.getPrincipal());
        model.addAttribute("spaceUsed", this.getSpaceUserd());
        return "users/edituser";
    }
	
	@RequestMapping(value = "/users/updateedituserfromuser", method = RequestMethod.POST)
	public String updateEditUserFromUser(@ModelAttribute("user") User user, Model model) {
		String urlRedirect ="";
		User existUser = this.userService.getUserByUserId(user.getUserid());
		if ((existUser!=null) && (existUser.getId()!=user.getId()))
		{			
			model.addAttribute("error", 1);
			urlRedirect = "redirect:/users/edituser/"+user.getId();							
		} else
		{
			User updatebleUser = this.userService.getUserById(user.getId());
			updatebleUser.setName(user.getName());
			updatebleUser.setSurname(user.getSurname());
			updatebleUser.setUserid(user.getUserid());
			this.userService.updateUser(updatebleUser);
			urlRedirect = "redirect:/users";
		}		
		return urlRedirect;	
	}
		
	@RequestMapping("/users/changeuserpassword/{id}")
    public String changeUserPasswordUser(@PathVariable("id") int id, Model model){ 
		UserPass userPass = new UserPass();
		userPass.setUserId(id);
		model.addAttribute("userPass", userPass);
        model.addAttribute("userNameAuthenticated", this.getPrincipal());
        model.addAttribute("spaceUsed", this.getSpaceUserd());
        return "users/changeuserpassword";
    }
		
	@RequestMapping(value = "/users/updateuserpassword", method = RequestMethod.POST)
	public String updateUserPassword(@ModelAttribute("userPass") UserPass userPass, Model model) throws Exception {
		User user = this.userService.getUserById(userPass.getUserId());		
		
		BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
		
		Boolean errorPasswordRepeted = !(userPass.getNewPassword().trim().equals(userPass.getNewPasswordRepeted().trim()));	
		
		String urlReturn = "redirect:/users/changeuserpassword/"+user.getId();	
		
		if (errorPasswordRepeted)
		{
			// Mal la repeticin de claves
			model.addAttribute("error", 1);
		} else 
		{
			// ok	
			user.setPassword(passwordEncoder.encode(userPass.getNewPassword().trim()));
			this.userService.updateUser(user);
			urlReturn = "redirect:/users";
		} 
		
		model.addAttribute("userNameAuthenticated", this.getPrincipal());
		model.addAttribute("spaceUsed", this.getSpaceUserd());
		return urlReturn;
	}
	
	@RequestMapping("/users/changepassword/{id}")
    public String changePasswordUser(@PathVariable("id") int id, Model model){ 
		UserPass userPass = new UserPass();
		userPass.setUserId(id);
		model.addAttribute("userPass", userPass);
        model.addAttribute("userNameAuthenticated", this.getPrincipal());
        model.addAttribute("spaceUsed", this.getSpaceUserd());
        return "users/changepassword";
    }
		
	@RequestMapping(value = "/users/updatepassword", method = RequestMethod.POST)
	public String updatePassword(@ModelAttribute("userPass") UserPass userPass, Model model) throws Exception {
		User user = this.userService.getUserById(userPass.getUserId());
				
		BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
		
		Boolean errorOldPassword = !passwordEncoder.matches(userPass.getOldPassword(), user.getPassword());
		Boolean errorPasswordRepeted = !(userPass.getNewPassword().trim().equals(userPass.getNewPasswordRepeted().trim()));
		
		String urlReturn = "redirect:/users/changepassword/"+user.getId();		
		
		if (errorOldPassword && errorPasswordRepeted)
		{
			// Mal clave de acceso y mal la repeticin de claves
			model.addAttribute("error", 3);
		} else if (!errorOldPassword && errorPasswordRepeted)
		{
			// Mal clave de acceso y mal la repeticin de claves
			model.addAttribute("error", 2);
		} else if (errorOldPassword && !errorPasswordRepeted)
		{
			// Mal clave de acceso y mal la repeticin de claves
			model.addAttribute("error", 1);
		} else
		{
			// ok	
			user.setPassword(passwordEncoder.encode(userPass.getNewPassword().trim()));
			this.userService.updateUser(user);
			urlReturn = "redirect:/logout";
		}
		
		model.addAttribute("userNameAuthenticated", this.getPrincipal());
		model.addAttribute("spaceUsed", this.getSpaceUserd());
		return urlReturn;	
	}
			
	@RequestMapping("/users/remove/{id}")
    public String removeUser(@PathVariable("id") int id){
		
        this.userService.removeUser(id);
        return "redirect:/users";
    }
	
	@RequestMapping(value = "/users/add", method = RequestMethod.GET)
	public String newUser(Model model) {
		model.addAttribute("user", new User());
		Map<Integer, String> roles = GetRoles();
					
		model.addAttribute("roleList", roles);	
		model.addAttribute("userNameAuthenticated", this.getPrincipal());
		model.addAttribute("spaceUsed", this.getSpaceUserd());
		return "users/add";
	}
	
	@RequestMapping(value = "/users/save", method = RequestMethod.POST)
	public String save(@ModelAttribute("user") User user, Model model) {
		User existUser = this.userService.getUserByUserId(user.getUserid());
		String urlRedirect ="";
		if ((existUser!=null))
		{
			model.addAttribute("error", 1);
			urlRedirect = "redirect:/users/add";	
		}
		else
		{
			BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
			String hashedPassword = passwordEncoder.encode(user.getPassword());
			user.setPassword(hashedPassword);
			this.userService.addUser(user);
			urlRedirect = "redirect:/users";
		}
				
		return urlRedirect;	
	}
	
	@RequestMapping("/users/edit/{id}")
    public String editUser(@PathVariable("id") int id, Model model){
        model.addAttribute("user", this.userService.getUserById(id));
        Map<Integer, String> roles = GetRoles();					
		model.addAttribute("roleList", roles);
        model.addAttribute("userNameAuthenticated", this.getPrincipal());
        model.addAttribute("spaceUsed", this.getSpaceUserd());
        return "users/edit";
    }
	
	@RequestMapping(value = "/users/updateedituser", method = RequestMethod.POST)
	public String updateEditUser(@ModelAttribute("user") User user, Model model) {
		String urlRedirect ="";
		User existUser = this.userService.getUserByUserId(user.getUserid());
		if ((existUser!=null) && (existUser.getId()!=user.getId()))
		{			
			model.addAttribute("error", 1);
			urlRedirect = "redirect:/users/edit/"+user.getId();							
		} else
		{
			User updatebleUser = this.userService.getUserById(user.getId());
			updatebleUser.setName(user.getName());
			updatebleUser.setSurname(user.getSurname());
			updatebleUser.setUserid(user.getUserid());
			updatebleUser.setRol(user.getRol());
			this.userService.updateUser(updatebleUser);
			urlRedirect = "redirect:/users";
		}
		return urlRedirect;	
	}
			
	private Map<Integer, String> GetRoles() {
		Map referenceData = new HashMap();
		Map<Integer,String> roles = new LinkedHashMap<Integer,String>();
		roles.put(-1, "");
		for (Rol rol : this.rolService.listRoles())
		{
			roles.put(rol.getId(), rol.getDescription());		
		}
		referenceData.get(roles);
		return roles;
	}
	
}
