package edu.ouc.mycloud.dao;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Repository;
import org.springframework.web.multipart.MultipartFile;

import edu.ouc.mycloud.model.File;
import edu.ouc.mycloud.model.Rol;
import edu.ouc.mycloud.model.Shared;
import edu.ouc.mycloud.model.StoredFile;
import edu.ouc.mycloud.model.User;

@Repository("FileDao")
public class FileDAOImpl implements FileDAO{

private static final Logger logger = LoggerFactory.getLogger(FileDAOImpl.class);
	
	@Autowired
	private SessionFactory sessionFactory;
	
	@Autowired
    private Environment environment;
				
	
	@Override
	public void setDetelePublicSharedFiles(int id) {
		Session session = this.sessionFactory.getCurrentSession();
		Query query = session.createQuery("delete Shared S WHERE S.file.id= :fileid");
		query.setParameter("fileid", id);
		query.executeUpdate();		
	}

	
	
	@SuppressWarnings("deprecation")
	@Override
	public Shared getFileByToken(String token) {
		Session session = this.sessionFactory.getCurrentSession();
		Criteria criteria = session.createCriteria(Shared.class);
		criteria.add(Restrictions.eq("token", token));		
		Shared shared = (Shared) criteria.uniqueResult();
		return shared;
	}
	
	@Override
	public long getSpaceUserd(User user) {
		Session session = this.sessionFactory.getCurrentSession();
		StringBuilder sb =  new StringBuilder();
		sb.append("select sum(S.filesize) from File F inner join F.storedfile S ");
		sb.append("inner join F.user U ");
		sb.append("where U.id = :id");
		
		List fileList = session.createQuery(sb.toString()).setInteger("id", user.getId()).list();;

		if (fileList.get(0)!=null)
		{
			return ((long) fileList.get(0))/(1024*1024);
		}
		else
		{
			return 0;
		}		
	}
	
	@Override
	public List<File> listFiles() {
		Session session = this.sessionFactory.getCurrentSession();
		List<File> fileList = session.createQuery("from File F WHERE F.parentfile is NULL ORDER BY F.isFolder DESC").list();
		for(File f : fileList){
			logger.info("File List::"+f);
		}
		return fileList;
	}
	
	@Override
	public List<File> listFiles(User user) {
		Session session = this.sessionFactory.getCurrentSession();
		List<File> fileList = session.createQuery("from File F WHERE (F.parentfile is NULL) AND (F.user.id = :id) ORDER BY F.isFolder DESC")
				.setInteger("id", user.getId()).list();
		for(File f : fileList){
			logger.info("File List::"+f);
		}
		return fileList;
	}

	@Override
	public List<File> listFilesByParentFile(int id) {
		Session session = this.sessionFactory.getCurrentSession();
		List<File> fileList = session.createQuery("from File F WHERE F.parentfile = :id  ORDER BY F.isFolder DESC")
				.setInteger("id", id).list();
		for(File f : fileList){
			logger.info("File List::"+f);
		}
		return fileList;
	}
	
	@Override
	public List<File> listFilesByParentFile(int id, User user) {
		Session session = this.sessionFactory.getCurrentSession();
		List<File> fileList = session.createQuery("from File F WHERE (F.parentfile = :id) AND (F.user.id = :userid)  ORDER BY F.isFolder DESC")
				.setInteger("id", id).setInteger("userid", user.getId()).list();
		for(File f : fileList){
			logger.info("File List::"+f);
		}
		return fileList;
	}

	@Override
	public void addFile(File file) {
		Session session = this.sessionFactory.getCurrentSession();
		session.save(file);
		logger.info("File saved successfully, File Details="+file);		
	}
	
	@Override
	public void addFile(File file, MultipartFile document) {
		Session session = this.sessionFactory.getCurrentSession();		
		try {			
			String storageDirectory = environment.getRequiredProperty("application.storage_path")+file.getStoredfile().getFilename();
			java.io.File convFile = new java.io.File(storageDirectory);		
			convFile.createNewFile();		
			FileOutputStream fos = new FileOutputStream(convFile); 
			fos.write(document.getBytes());;
			fos.close();			
			session.save(file);	    
			} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			}
		logger.info("File saved successfully, File Details="+file);			
	}

	@Override
	public List<File> listParentByParentFile(int id) {
		Session session = this.sessionFactory.getCurrentSession();
		List<File> fileList = new ArrayList<File>();
		
		File file;
		
		file = (File) session.load(File.class, new Integer(id));		
		while (file.getParentfile()!=null) {
			file = (File) session.load(File.class, new Integer(id));
			fileList.add(file);
			file = (File) session.load(File.class, new Integer(id));
		}		
		return fileList;
	}

	@Override
	public File getFileById(int id) {
		Session session = this.sessionFactory.getCurrentSession();		
		File file = (File) session.load(File.class, new Integer(id));
		logger.info("File loaded successfully, File details="+file);
		return file;
	}

	@Override
	public void updateFile(File file) {
		Session session = this.sessionFactory.getCurrentSession();
		session.update(file);
		logger.info("File updated successfully, File Details="+file);		
	}

	@Override
	public boolean removeFile(int id) {
		boolean opSucess = false;
		Session session = this.sessionFactory.getCurrentSession();
		File file = (File) session.load(File.class, new Integer(id));		
		if(null != file){
			
			List<File> listChilds = listFilesByParentFile(id);
			if (listChilds.size()==0)
			{							
				StoredFile storedFile=null;			
				if (file.getStoredfile()!=null)
				{		
					setDetelePublicSharedFiles(file.getId());
					storedFile =  (StoredFile) session.load(StoredFile.class, file.getStoredfile().getId());
					String storageDirectory = environment.getRequiredProperty("application.storage_path")+file.getStoredfile().getFilename();
					java.io.File convFile = new java.io.File(storageDirectory);	
					convFile.delete();
				}
				session.delete(file);
			
				if (storedFile!=null)
				{
					session.delete(storedFile);
				}
				opSucess = true;
			}
			else
			{
				opSucess = false;
			}
			
		}
		logger.info("File deleted successfully, File details="+file);	
		return opSucess;
	}

	@Override
	public long getNumberOfVirtualFiles() {
		Session session = this.sessionFactory.getCurrentSession();
		String hql = "select count(*) from File F where F.isFolder = false";
		
		List fileList = session.createQuery(hql).list();;
		if (fileList.get(0)!=null)
		{
			return (long) fileList.get(0);
		}
		else
		{
			return 0;
		}	
	}

	@Override
	public long getNumberOfFiles() {
		Session session = this.sessionFactory.getCurrentSession();
		String hql = "select count(*) from StoredFile S";
		
		List fileList = session.createQuery(hql).list();;
		if (fileList.get(0)!=null)
		{
			return (long) fileList.get(0);
		}
		else
		{
			return 0;
		}	
	}

	@Override
	public InputStream getStoredPhysicalFile(int id) {
		Session session = this.sessionFactory.getCurrentSession();		
		File file = (File) session.load(File.class, new Integer(id));
		
		StringBuilder sb = new StringBuilder();
		sb.append(environment.getRequiredProperty("application.storage_path"));
		sb.append("\\");
		sb.append(file.getStoredfile().getFilename());
		
		java.io.File convFile = new java.io.File(sb.toString());
		
		System.out.println(sb.toString());
		
		InputStream inputStream=null;
		try {
			inputStream = new java.io.BufferedInputStream(new java.io.FileInputStream(convFile));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return inputStream;
	}

	@Override
	public void setSharedFile(Shared shared) {
		Session session = this.sessionFactory.getCurrentSession();
		session.save(shared);
		logger.info("Shared file saved successfully, File Details="+shared);				
	}

		
}