package edu.ouc.mycloud.service;

import java.io.InputStream;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import edu.ouc.mycloud.dao.FileDAO;
import edu.ouc.mycloud.model.File;
import edu.ouc.mycloud.model.Shared;
import edu.ouc.mycloud.model.StoredFile;
import edu.ouc.mycloud.model.User;

@Service("storageService")
public class StorageServiceImpl implements StorageService {

	@Autowired
	private FileDAO fileDAO;
	
	@Transactional
	public List<File> listFiles(User user) {
		return this.fileDAO.listFiles(user);	
	}
	
	@Transactional
	public List<File> listFiles() {
		return this.fileDAO.listFiles();	
	}

	@Transactional
	public List<File> listFilesByParentFile(int id) {
		return this.fileDAO.listFilesByParentFile(id);
	}
	@Transactional
	public List<File> listFilesByParentFile(int id, User loggedUser) {
		return this.fileDAO.listFilesByParentFile(id, loggedUser);
	}

	@Transactional
	public void addFile(File file) {
		this.fileDAO.addFile(file);			
	}
	
	@Transactional
	public void addFile(File file, MultipartFile document) {
		this.fileDAO.addFile(file, document);		
	}

	@Transactional
	public File getFileById(int id) {
		return this.fileDAO.getFileById(id);
	}
	
	@Transactional
	public void updateFile(File file) {
		this.fileDAO.updateFile(file);		
	}

	@Transactional
	public boolean removeFile(int id) {
		return this.fileDAO.removeFile(id);			
	}
	
	@Transactional
	public void addStorageFile(File file) {
		this.fileDAO.addFile(file);			
	}

	@Transactional
	public long getSpaceUserd(User user) {		
		return this.fileDAO.getSpaceUserd(user);
	}

	@Transactional
	public long getNumberOfVirtualFiles() {
		return this.fileDAO.getNumberOfVirtualFiles();
	}

	@Transactional
	public long getNumberOfFiles() {
		return this.fileDAO.getNumberOfFiles();
	}

	@Transactional
	public InputStream getStoredPhysicalFile(int id) {
		return this.fileDAO.getStoredPhysicalFile(id);
	}

	@Transactional
	public void setSharedFile(Shared shared) {
		this.fileDAO.setSharedFile(shared);
	}

	@Transactional
	public Shared getFileByToken(String token) {
		return this.fileDAO.getFileByToken(token);
	}
}
